﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

internal class Program
{
    static void Main(string[] args)
    {
        ThreadPool.QueueUserWorkItem(ServerProc);

        while (true)
        {
            ClientProc();

            Console.WriteLine("Press any key to send again....");
            Console.ReadLine();
        }
    }

    static void ClientProc()
    {
        HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create("http://127.0.0.1:9982/");
        httpWebRequest.Method = "POST";

        string data = new string('c', 51);

        byte[] sendBuf = Encoding.UTF8.GetBytes(data);
        using (Stream dataStream = httpWebRequest.GetRequestStream())
        {
            dataStream.Write(sendBuf, 0, sendBuf.Length);
        }

        try
        {
            using (HttpWebResponse resp = (HttpWebResponse)httpWebRequest.GetResponse())
            {
                using (StreamReader streamReader = new StreamReader(resp.GetResponseStream()))
                {
                    string responseText = streamReader.ReadToEnd();

                    if (resp.StatusCode == HttpStatusCode.OK)
                    {
                        Console.WriteLine($"[Client] {responseText}");
                    }
                }
            }
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
    }

    static void ServerProc(object state)
    {
        using (Socket serverSocket = new Socket(SocketType.Stream, ProtocolType.Tcp))
        {
            serverSocket.Bind(new IPEndPoint(IPAddress.IPv6Any, 9982));
            serverSocket.Listen(5);

            while (true)
            {
                Socket socket = serverSocket.Accept();
                ThreadPool.QueueUserWorkItem(processChildSocket, socket);
            }
        }
    }

    static void processChildSocket(object state)
    {
        using (Socket child = state as Socket)
        {
            string txt = ReceiveAll(child);

            Console.WriteLine($"[Server] {txt}");

            string body = "test";
            string header = "HTTP/1.1 200 OK\r\nContent-Type: application/json; charset=UTF-8\r\n\r\n";
            byte[] responseBuf = Encoding.UTF8.GetBytes(header + body);

            child.Send(responseBuf);
        }
    }

    private static string ReceiveAll(Socket child)
    {
        byte[] buf = new byte[8192];

        int contentLengthPos = -1;
        int bodyStart = -1;
        string text = "";
        string contentLengthHeader = "Content-Length";
        int readLen = 0;

        // 대개의 경우 한 번의 Read에 HTTP Header가 읽힘
        // 그래도 HTTP Header가 큰 경우가 있을 수 있다는 사실!
        {
            readLen = child.Receive(buf);

            text = Encoding.UTF8.GetString(buf, 0, readLen);
            contentLengthPos = text.IndexOf(contentLengthHeader, 0, StringComparison.OrdinalIgnoreCase);
            bodyStart = text.IndexOf("\r\n\r\n", contentLengthPos);
        }

        int contentLength = 0;

        if (contentLengthPos > 0)
        {
            int startPos = contentLengthPos + contentLengthHeader.Length;
            int endPos = text.IndexOf("\r\n", startPos);
            if (endPos != -1)
            {
                string contentLengthText = text.Substring(startPos, endPos - startPos).Trim(' ', ':');
                contentLength = int.Parse(contentLengthText);
            }
        }

        int remains = (bodyStart + 4 + contentLength) - readLen;

        if (remains == 0)
        {
            return text;
        }

        StringBuilder sb = new StringBuilder();

        while (remains > 0)
        {
            int len = child.Receive(buf);
            if (len <= 0)
            {
                break;
            }

            remains -= len;

            sb.Append(Encoding.UTF8.GetString(buf, 0, len));
        }

        Console.WriteLine("****** Server-splitted");
        return text + sb.ToString();
    }
}
